'''
Created on 31 Aug 2009

@author: jamie
'''

import Box2D
import DebugDraw

class CollisionGroup:
    icarus = 0x0003
    icarus_wings = 0x0001
    icarus_feet = 0x0002
    bird = 0x0004
    birdsensor = 0x0008
    wall = 0x0010
    trigger = 0x0020
    wind = 0x0040
    thermal = 0x0080
    bullet = 0x0100
    feather = 0x0200
    particle = 0x0400

class Actor(object):
    body = None
    debugDraw = None
    isAlive = True
    world = None
    
    def __init__(self, body, debugDraw, world):
        self.body = body        
        self.debugDraw = debugDraw
        self.world = world
        
        self.body.SetUserData(self)

    def Update(self, keys):
        # Do nothing
        pass
    
    def Render(self, doDebugDraw):
        # By default, just do debug rendering of any shapes attached to the body.
        # Subclasses can override this
        if doDebugDraw:
            if self.body is None:
                return
            for shape in self.body.GetShapeList(True):
                if isinstance(shape, Box2D.b2CircleShape):
                    center = self.body.GetWorldPoint(shape.GetLocalPosition())
                    radius = shape.GetRadius()
                    color = Box2D.b2Color(0.8, 0.1, 0.1)
                    self.debugDraw.DrawCircle(center, radius, color)
                elif isinstance(shape, Box2D.b2PolygonShape):
                    vertices = [self.body.GetWorldPoint(p).tuple() for p in shape.getVertices_b2Vec2()]
                    vertexCount = shape.GetVertexCount()
                    color = Box2D.b2Color(0.8, 0.1, 0.1)
                    self.debugDraw.DrawPolygon(vertices, vertexCount, color)

    def Kill(self):
        if self.isAlive:
            self.isAlive = False
            self.world.DestroyBody(self.body)
            self.body = None
     