import Actor
import Box2D
import pygame
import DebugDraw
import IcarusActor
import WallActor
import BulletActor
import random

class BirdActor(Actor.Actor):
    
    shotCallback = None
    bodyShape = None
    sensorShape = None
    frames = []
    skiprate = 3
    frameskip = skiprate
    frameno = 1
    renderpos = (0, 0)
    spritewidth = 0
    spriteheight = 0
    
    def __init__(self, world, debugDraw, position, frames, shotCallback):    
        
        self.shotCallback = shotCallback
        self.frames = frames
        self.spritewidth = self.frames[0].get_width() / 2
        self.spriteheight = self.frames[0].get_height() / 2
        self.frameno = random.randint(1, len(frames))        
        
        # set up a body/shape for the bird (circle will do)         
        birdBodyDef = Box2D.b2BodyDef()
        birdBodyDef.position = position
        birdBodyDef.fixedRotation = True
        birdBodyDef.linearDamping = 50
        birdBodyDef.allowSleep = True
        birdBody = world.CreateBody(birdBodyDef)
        
        birdShapeDef = Box2D.b2CircleDef()
        birdShapeDef.radius = 1.5
        birdShapeDef.density = 0.25
        birdShapeDef.filter.categoryBits = Actor.CollisionGroup.bird
        self.bodyShape = birdBody.CreateShape(birdShapeDef)
        
        birdSensorDef = Box2D.b2CircleDef()
        birdSensorDef.radius = 5
        birdSensorDef.isSensor = True
        birdSensorDef.filter.categoryBits = Actor.CollisionGroup.birdsensor

        self.sensorShape = birdBody.CreateShape(birdSensorDef)

        birdBody.SetMassFromShapes()       
        
        # Call the base constructor, with our body
        super(BirdActor, self).__init__(birdBody, debugDraw, world)
        
    def HandleContact(self, contact):
        if self.isAlive == False:
            return
        
        if contact.shape1.GetBody().userData != self:
            print "Error: BirdActor.HandleContact called, but I am not shape1 owner"
            return

        actor2 = contact.shape2.GetBody().userData
        
        if isinstance(actor2, WallActor.WallActor) | isinstance(actor2, IcarusActor.IcarusActor) | isinstance(actor2, BirdActor):
            # Fly away from walls, icarus and other birds
            p = self.body.GetWorldPoint((0.0, 0.0))
            self.body.ApplyForce(contact.normal * -2000, p)
        elif isinstance(actor2, BulletActor.BulletActor):
            # kill the bird, and spawn some feathers
            self.shotCallback(self)
            self.Kill()

    def Update(self, keys):
        self.renderpos = (self.body.position.x, #- self.spritewidth,
                          self.body.position.y) # - self.spriteheight)
        
    def Render(self, doDebugDraw):       
        self.frameskip -= 1
        currentframe = self.frames[self.frameno % len(self.frames)]
        self.debugDraw.DrawImage(currentframe, self.renderpos, self.spritewidth, self.spriteheight)
        if self.frameskip == 0:                        
            self.frameno += 1
            self.frameskip = self.skiprate  
        super(BirdActor, self).Render(doDebugDraw)
        
        
        
        