import Box2D
import Actor
import WindActor
import pygame

class BulletActor(Actor.Actor):
    
    framesToLive = 250
    lifeTime = 0
    speedMultiplier = 5
    bulletImage = None
     
    def __init__(self, world, debugDraw, position, velocity):
      
        bd = Box2D.b2BodyDef()        
        bd.position = position
        bd.isBullet = True
        bulletBody = world.CreateBody(bd)
        bulletBody.SetLinearVelocity(velocity * self.speedMultiplier)

        sd = Box2D.b2CircleDef()       
        sd.radius = 0.3
        sd.density = 20.0
        sd.restitution = 0.1
        sd.filter.categoryBits = Actor.CollisionGroup.bullet
        sd.filter.maskBits = Actor.CollisionGroup.bird | Actor.CollisionGroup.wall | Actor.CollisionGroup.wind
        
        bulletBody.CreateShape(sd)
        bulletBody.SetMassFromShapes()
        
        self.bulletImage = pygame.Surface((3,3))
        self.bulletImage.fill((200,80,80))
        
        super(BulletActor, self).__init__(bulletBody, debugDraw, world)
        
    def HandleContact(self, contact):
#        print "bullet contact %s" % contact.actor2
        
        if self.isAlive == False:
            return

        if contact.shape1.GetBody().userData != self:
            print "Error: BulletActor.HandleContact called, but I am not shape1 owner"
            return

        actor2 = contact.shape2.GetBody().userData
        
        if isinstance(actor2, WindActor.WindActor):
            actor2.HandleTouchedByBullet(self)

    def Update(self, keys):
        if self.body is None:
            return
        self.lifeTime += 1
        if self.lifeTime > self.framesToLive:
            self.Kill()
            
    def Render(self, doDebugDraw):
        if self.body is None:
            return
        self.debugDraw.DrawImage(self.bulletImage, self.body.position.tuple())        
        super(BulletActor, self).Render(doDebugDraw)