'''
Created on 1 Sep 2009

@author: jamie
'''

import Actor
import Box2D
import pygame
import DebugDraw
import WallActor
import TriggerActor
import WindActor
import ThermalActor
import FeatherActor
import random

class IcarusActor(Actor.Actor):
    collectedFeatherCallback = None
    
    feetShape = None
    wingShape = None
    
    # Counts frames for icarus falling, when the user can't apply controls
    fallingTimer = 0

    # Number of frames for a complete wing-beat (number of animation frames * frameskip)
    wingbeatPeriod = 51 
    # Fraction of wing-beat period that a thrust is active
    wingbeatDutyCycle = 0.3
    # Wing-beat frame counter
    wingbeatCounter = 0
    
    # animation stuff
    frames = []
    framesLeft = []
    framesRight = []
    skiprate = 3
    frameskip = skiprate
    frameno = 1
    renderpos = (0, 0)
    spritewidth = 0
    spriteheight = 0
    isAnimating = True
    
    # Tracks how many feathers icarus has collected this level 
    numFeathers = 0

    soundbank = None
    
    burningTimer = 0
      
    def __init__(self, world, debugDraw, position, collectedFeatherCallback, framesRight, framesLeft, soundbank):
        self.collectedFeatherCallback = collectedFeatherCallback
        self.framesLeft = framesLeft
        self.framesRight = framesRight
        self.frames = framesRight
        self.spritewidth = self.frames[0].get_width() / 2
        self.spriteheight = self.frames[0].get_height() / 2
        self.frameno = random.randint(1, len(self.frames) - 1)     
        self.soundbank = soundbank
        
        icarusBodyDef = Box2D.b2BodyDef()
        icarusBodyDef.position = position
        icarusBodyDef.fixedRotation = True
        icarusBodyDef.linearDamping = 0.7
        icarusBody = world.CreateBody(icarusBodyDef)

        feetShapeDef = Box2D.b2PolygonDef() 
        feetShapeDef.density = 1
        feetShapeDef.SetAsBox(0.5, 0.2)
        feetShapeDef.filter.categoryBits = Actor.CollisionGroup.icarus_feet
        feetShapeDef.filter.maskBits = Actor.CollisionGroup.wall 
        self.feetShape = icarusBody.CreateShape(feetShapeDef)

        wingShapeDef = Box2D.b2CircleDef()
        wingShapeDef.radius = 1 
        wingShapeDef.density = 1
        wingShapeDef.localPosition = (0, 1)
        wingShapeDef.filter.categoryBits = Actor.CollisionGroup.icarus_wings
        self.wingShape = icarusBody.CreateShape(wingShapeDef)
        
        icarusBody.SetMassFromShapes()

        # Call the base constructor, with our body
        super(IcarusActor, self).__init__(icarusBody, debugDraw, world)
        
    def StartFalling(self):
        self.soundbank['headhit'].play()
        self.fallingTimer = 70
        self.wingbeatCounter = 0
        
    def IsFalling(self):
        return self.fallingTimer > 0
    
    def UpdateFallingState(self):
        if self.IsFalling():
            self.fallingTimer = self.fallingTimer - 1
    
    def IsWingThrustAvailable(self):
        return self.wingbeatCounter < (self.wingbeatPeriod * self.wingbeatDutyCycle)
    
    def UpdateWingBeatState(self):
        if self.wingbeatCounter > 0:
            # A wing beat is currently in progress, so advance it
            self.wingbeatCounter = self.wingbeatCounter + 1
            if (self.wingbeatCounter >= self.wingbeatPeriod):
                self.wingbeatCounter = 0
    
    def ApplyWingThrust(self, direction, goingup=False):
        p = self.body.GetWorldPoint((0.0, 0.0))
        self.body.ApplyForce(direction, p)
        
        if (self.wingbeatCounter == 0):
            # No wing beat is currently active, so start a new one            
            if goingup: self.soundbank['wingbeat-' + str(random.randint(1,4))].play()
            self.wingbeatCounter = 1
            
    def HandleContact(self, contact):
        if contact.shape1.GetBody().userData != self:
            print "Error: IcarusActor.HandleContact called, but I am not shape1 owner"
            return

        actor2 = contact.shape2.GetBody().userData
        
        if contact.shape1 == self.feetShape:
            # feet have touched something
            # if it is a wall, can stop moving and go to sleep
            if isinstance(actor2, WallActor.WallActor):
                # Stop moving and go to sleep - as we have landed on a ledge/floor
                self.body.SetLinearVelocity(Box2D.b2Vec2(0, 0))
                self.isAnimating = False                
                #self.body.PutToSleep()
        elif contact.shape1 == self.wingShape:
            # if it is a wall, push away slightly and start falling
            if isinstance(actor2, WallActor.WallActor):
                p = self.body.GetWorldPoint((0.0, 0.0))
                self.body.ApplyForce(contact.normal * -300, p)
                # Make icarus fall for a while
                self.StartFalling()
            elif isinstance(actor2, TriggerActor.TriggerActor):
                actor2.HandleTouchedByIcarus(self)
            elif isinstance(actor2, WindActor.WindActor):
                actor2.HandleTouchedByIcarus(self)
            elif isinstance(actor2, ThermalActor.ThermalActor):
                actor2.HandleTouchedByIcarus(self)
            elif isinstance(actor2, FeatherActor.FeatherActor):
                if actor2.isAlive:
                    if actor2.CanBeCollected():
                        self.numFeathers+=1
                        self.collectedFeatherCallback(actor2)
                        actor2.Kill()

    def Update(self, keys):        
        
        self.renderpos = (self.body.position.x, self.body.position.y)
        
        
        self.UpdateWingBeatState()
        
        if self.IsFalling():
            # Icarus is falling
            self.UpdateFallingState()
        else:
            if keys[pygame.K_w]:
                if self.IsWingThrustAvailable():                    
                    if self.body.GetLinearVelocity().y < 10:
                        self.isAnimating = True
                        self.ApplyWingThrust((0, 300 + (self.numFeathers * 20)), True)                
            if keys[pygame.K_a]:                
                if self.body.GetLinearVelocity().x > -10:
                    self.ApplyWingThrust((-100 - (self.numFeathers * 20), 0))
                    self.frames = self.framesLeft
            elif keys[pygame.K_d]:                
                if self.body.GetLinearVelocity().x < 10:
                    self.ApplyWingThrust((100 + (self.numFeathers * 20), 0))
                    self.frames = self.framesRight
            
        if self.burningTimer != 0:
            self.burningTimer -= 1
        
    def Render(self, doDebugDraw):                
        
        currentframe = self.frames[self.frameno % len(self.frames)]
        self.debugDraw.DrawImage(currentframe, self.renderpos, self.spritewidth, self.spriteheight+10)
        if self.isAnimating:
            self.frameskip -= 1
            if self.frameskip == 0:                        
                self.frameno += 1
                if self.frameno == len(self.frames):
                    self.isAnimating = False
                    self.frameno = 0
                self.frameskip = self.skiprate  
        
        super(IcarusActor, self).Render(doDebugDraw)
        