import Actor
import Box2D
import pygame
import DebugDraw
import IcarusActor
import WallActor
import WindActor
import ThermalActor
import random
import math

class ParticleActor(Actor.Actor):
    lifetimeTimer = 10
    color = None
    parentActor = None
    spriteSurface = None
    
    def __init__(self, world, debugDraw, position, color, parentActor):    
        self.color = color
        self.parentActor = parentActor
        self.spriteSurface = pygame.Surface((3,3))
        self.spriteSurface.fill(color)
        
        # set up a body/shape for the particle         
        particleBodyDef = Box2D.b2BodyDef()
        particleBodyDef.position = position
        particleBodyDef.fixedRotation = True
        particleBodyDef.linearDamping = 0.2
        particleBody = world.CreateBody(particleBodyDef)

        particleShapeDef = Box2D.b2CircleDef()
        particleShapeDef.radius = 1
        particleShapeDef.density = 0.3
        particleShapeDef.friction = 1
        particleShapeDef.restitution = 1
        particleShapeDef.isSensor = True
        particleShapeDef.filter.categoryBits = 0 # Don't want the particle to collide with anything 
        particleShapeDef.filter.categoryBits = Actor.CollisionGroup.particle
        particleShapeDef.filter.maskBits = Actor.CollisionGroup.wind | Actor.CollisionGroup.thermal
        particleBody.CreateShape(particleShapeDef)
        
        particleBody.SetMassFromShapes()
        
        # Call the base constructor, with our body
        super(ParticleActor, self).__init__(particleBody, debugDraw, world)
        
    def HandleContact(self, contact):
        if self.isAlive == False:
            return

        if contact.shape1.GetBody().userData != self:
            print "Error: ParticleAator.HandleContact called, but I am not shape1 owner"
            return

        actor2 = contact.shape2.GetBody().userData
        
        if isinstance(actor2, WindActor.WindActor):
            actor2.HandleTouchedByParticle(self)
        elif isinstance(actor2, ThermalActor.ThermalActor):
            actor2.HandleTouchedByParticle(self)
        
        if actor2 == self.parentActor:
            # as long as particle is touching the parent actor, keep living
            self.lifetimeTimer = 2

    def Update(self, keys):
        if self.lifetimeTimer == 0:
            self.Kill()
        else:
            self.lifetimeTimer-=1

            # particle should be minimally affected by gravity
            p = self.body.GetWorldPoint((0.0, 0.0))
            weight = self.body.GetMass() * self.world.GetGravity()
            self.body.ApplyForce(-weight, p)
        
    def Render(self, doDebugDraw):    
        if self.body is None:
            return
        
        #self.debugDraw.DrawPoint(self.body.GetPosition(), 2, self.color)
        self.debugDraw.DrawImage(self.spriteSurface, self.body.GetPosition().tuple())

        super(ParticleActor, self).Render(doDebugDraw)
