'''
Created on 2 Sep 2009

@author: jamie
'''
import Actor
import Box2D
import DebugDraw
import math
import random
import ParticleActor

class ThermalActor(Actor.Actor):
    
    lowestPoint = None
    highestPoint = None
    particleSpawnTimer = 0
    actors = None
        
    def __init__(self, pos, points, world, debugDraw, actors):        
        ''' Create a non-collidable thermal actor from a list of points'''
        
        self.actors = actors
        
        thermalBodyDef = Box2D.b2BodyDef()
        thermalBodyDef.position = pos        
        thermalBody = world.CreateBody(thermalBodyDef)        
        
        thermalShapeDef = Box2D.b2PolygonDef()
        thermalShapeDef.vertexCount = len(points)
        thermalShapeDef.setVertices(points)
        thermalShapeDef.isSensor = 1
        thermalShapeDef.filter.categoryBits = Actor.CollisionGroup.thermal
        thermalShapeDef.filter.maskBits = Actor.CollisionGroup.icarus | Actor.CollisionGroup.bird | Actor.CollisionGroup.feather | Actor.CollisionGroup.particle
                
        thermalBody.CreateShape(thermalShapeDef)
        
        #find the lowest and highest points - these will be our offsets for strength calcs
        self.lowestPoint = Box2D.b2Vec2(points[0])
        self.highestPoint = Box2D.b2Vec2(points[0])
        for point in points[1:]:
            if point[1] < self.lowestPoint.y:
                self.lowestPoint = Box2D.b2Vec2(point)        
            if point[1] > self.highestPoint.y:
                self.highestPoint = Box2D.b2Vec2(point)        

        # Call the base constructor, with our body
        super(ThermalActor, self).__init__(thermalBody, debugDraw, world)

    def CalculateStrengthAtPoint(self, point):
        return (self.highestPoint.y - point.y) / (self.highestPoint.y - self.lowestPoint.y)
        
    def Update(self, keys):
        if self.particleSpawnTimer == 0:
            self.particleSpawnTimer = 40
            spawnPosition = self.lowestPoint + (random.uniform(-5, 5), 0)
            particle = ParticleActor.ParticleActor(self.world, self.debugDraw, spawnPosition, (200, 150, 50), self)

            p = particle.body.GetWorldPoint((0, 0))
            strength = self.CalculateStrengthAtPoint(spawnPosition) * 5
            particle.body.ApplyImpulse((random.uniform(-5, 5), strength), spawnPosition)

            self.actors.append(particle)
        else:
            self.particleSpawnTimer-=1

    def HandleTouchedByIcarus(self, icarus):
        p = icarus.body.GetWorldPoint((0.0, 0.0))
        #strength falls of from max at bottom to zero at top
        #strength = (self.highestPoint.y - p.y) / (self.highestPoint.y - self.lowestPoint.y) * 200
        strength = self.CalculateStrengthAtPoint(p) * 100
        icarus.body.ApplyForce((0, strength), p)

    def HandleTouchedByFeather(self, feather):
        p = feather.body.GetWorldPoint((0.0, 0.0))
        #strength falls of from max at bottom to zero at top
        #strength = (self.highestPoint.y - p.y) / (self.highestPoint.y - self.lowestPoint.y) * 50
        strength = self.CalculateStrengthAtPoint(p) * 100
        feather.body.ApplyForce((0, strength), p)

    def HandleTouchedByParticle(self, particle):
        p = particle.body.GetWorldPoint((0.0, 0.0))
        #strength = (self.highestPoint.y - p.y) / (self.highestPoint.y - self.lowestPoint.y) * 50
        strength = self.CalculateStrengthAtPoint(p) * 10
        particle.body.ApplyForce((0, strength), p)

    def Render(self, doDebugDraw):
        # draw a circle around the highest point
        #self.debugDraw.DrawCircle(self.highestPoint, 1, (0,1,0))
        # draw a circle around the lowest point
        #self.debugDraw.DrawCircle(self.lowestPoint, 1, (0,0,1))
        super(ThermalActor, self).Render(doDebugDraw)
