'''
Created on 2 Sep 2009

@author: jamie
'''
import Actor
import Box2D
import DebugDraw
import ParticleActor
import math
import random

class WindActor(Actor.Actor):
    
    windShape = None
    windDirection = None
    windStrength = None
    windWidth = 0
    particleSpawnTimer = 0
    actors = None
        
    def __init__(self, points, width, strength, world, debugDraw, actors):        
        ''' Create a non-collidable wind actor from a list of points'''
        
        # points should contains exactly two points, describing the start and end of the wind direction
        if len(points) != 2:
            print "ERROR: WindActor with incorrect point count %d" % len(points)
            return
        
        self.windStrength = strength
        self.windWidth = width
        self.actors = actors
        
        self.windDirection = Box2D.b2Vec2(points[1][0] - points[0][0], points[1][1] - points[0][1])
        length = self.windDirection.Normalize()
        a = Box2D.b2Vec2(0, width /2)
        b = Box2D.b2Vec2(0, -width /2)
        c = Box2D.b2Vec2(length, -width /2)
        d = Box2D.b2Vec2(length, width /2)
        
            
        windBodyDef = Box2D.b2BodyDef()
        windBodyDef.position = points[0]
        windBodyDef.angle = math.atan2(self.windDirection.y, self.windDirection.x)        
        windBody = world.CreateBody(windBodyDef)        
        
        
        windShapeDef = Box2D.b2PolygonDef()
        windShapeDef.vertexCount = 4
        windShapeDef.setVertices((a, b, c, d))
        windShapeDef.isSensor = True
        windShapeDef.filter.categoryBits = Actor.CollisionGroup.wind
        windShapeDef.filter.maskBits = Actor.CollisionGroup.icarus | Actor.CollisionGroup.bird | Actor.CollisionGroup.feather | Actor.CollisionGroup.bullet | Actor.CollisionGroup.particle
                
        self.windShape = windBody.CreateShape(windShapeDef)
        
        # Call the base constructor, with our body
        super(WindActor, self).__init__(windBody, debugDraw, world)
        
    def Update(self, keys):
        if self.particleSpawnTimer == 0:
            self.particleSpawnTimer = 40
            spawnPosition = self.body.GetWorldPoint((0.2, random.uniform(-self.windWidth /2, self.windWidth / 2)))
            particle = ParticleActor.ParticleActor(self.world, self.debugDraw, spawnPosition, (128, 128, 200), self)
            
            p = particle.body.GetWorldPoint((0.0, 0.0))
            particle.body.ApplyImpulse(self.windDirection * self.windStrength * 0.05, p)
            
            self.actors.append(particle)
        else:
            self.particleSpawnTimer-=1

    def HandleTouchedByIcarus(self, icarus):
        p = icarus.body.GetWorldPoint((0.0, 0.0))
        icarus.body.ApplyForce(self.windDirection * self.windStrength, p)

    def HandleTouchedByFeather(self, feather):
        p = feather.body.GetWorldPoint((0.0, 0.0))
        feather.body.ApplyForce(self.windDirection * self.windStrength * 0.3, p)

    def HandleTouchedByBullet(self, bullet):
        p = bullet.body.GetWorldPoint((0.0, 0.0))
        bullet.body.ApplyForce(self.windDirection * self.windStrength * 8, p)

    def HandleTouchedByParticle(self, particle):
        p = particle.body.GetWorldPoint((0.0, 0.0))
        particle.body.ApplyForce(self.windDirection * self.windStrength * 0.05, p)
